import unittest
import mock
import pandas as pd
import cloudy_warehouses


class TestWriteSnowflake(unittest.TestCase):
    """Unittests for SnowflakeWriter."""

    def test_write_snowflake(self):

        df = pd.DataFrame.from_dict({'COL_1': ['hello', 'there'], 'COL_2': ['general', 'hello']})

        mock_write_snowflake = mock.create_autospec(df.cloudy_warehouses.write_snowflake, return_value=True)

        self.assertEqual(mock_write_snowflake(database='a database', schema='a schema', table='a table',
                                              sf_username='a name', sf_password='a pass', sf_account='an account'),
                         True)

        mock_write_snowflake.assert_called_with(database='a database', schema='a schema', table='a table',
                                                sf_username='a name', sf_password='a pass', sf_account='an account')


from cloudy_warehouses.snowflake_objects.snowflake_object import SnowflakeObject


class SnowflakeReader(SnowflakeObject):
    """Contains read and list_tables methods."""

    def read(self, database: str, schema: str, table: str, sf_username: str = None, sf_password: str = None, sf_account: str = None):
        """reads a table from Snowflake and returns a pandas dataframe of that table."""

        try:
            # configure and connect to Snowflake
            self.initialize_snowflake(
                database=database,
                schema=schema,
                sf_username=sf_username,
                sf_password=sf_password,
                sf_account=sf_account
            )

            # calls function to return data in a Snowflake table as a pandas dataframe
            df = self.get_pandas_dataframe(
                connection=self.connection,
                database=database,
                schema=schema,
                table=table
            )

        # catch and log error
        except Exception as e:
            self.log_message = e
            self._logger.error(self.log_message)
            return False

        finally:
            # close connection and cursor
            if self.connection:
                self.connection.close()
            if self.cursor:
                self.cursor.close()

        # log successful clone
        self.log_message = f"Successfully read from {database}.{schema}.{table}"
        self._logger.info(self.log_message)
        return df

    def list_tables(self, database: str, sf_username: str = None, sf_password: str = None, sf_account: str = None):
        """lists all tables in the specified Snowflake database. The list is returned as a pandas dataframe."""

        try:
            # configure and connect to Snowflake
            self.initialize_snowflake(
                database=database,
                sf_username=sf_username,
                sf_password=sf_password,
                sf_account=sf_account
            )

            # calls function to return data in a Snowflake table as a pandas dataframe
            df = self.get_snowflake_tables(
                connection=self.connection,
                database=database
                )

        # catch and log error
        except Exception as e:
            self.log_message = e
            self._logger.error(self.log_message)
            return False

        finally:
            # close connection and cursor
            if self.connection:
                self.connection.close()
            if self.cursor:
                self.cursor.close()

        # log successful clone
        self.log_message = f"Successfully listed tables from {database}"
        self._logger.info(self.log_message)
        return df

    def get_pandas_dataframe(self, connection, database, schema, table):
        """Reads data from a Snowflake table as a pandas dataframe."""
        self.cursor = connection.cursor()
        self.cursor.execute(f'select * from {database}.{schema}.{table}')
        df = self.cursor.fetch_pandas_all()

        return df

    def get_snowflake_tables(self, connection, database):
        """Reads data from a Snowflake table as a pandas dataframe."""
        self.cursor = connection.cursor()
        self.cursor.execute(
            f"SELECT TABLE_CATALOG, TABLE_SCHEMA, TABLE_NAME FROM {database}.INFORMATION_SCHEMA.TABLES;")
        tables = self.cursor.fetch_pandas_all()

        return tables




import pandas as pd
from snowflake.sqlalchemy import URL
from sqlalchemy import create_engine
from snowflake.connector.pandas_tools import pd_writer
from cloudy_warehouses.snowflake_objects.snowflake_object import SnowflakeObject


@pd.api.extensions.register_dataframe_accessor("cloudy_warehouses")
class SnowflakeWriter(SnowflakeObject):
    """Writer Object: contains write_snowflake and create_snowflake methods."""

    # True when write_snowflake successfully runs
    write_success = False
    # SQL Alchemy Engine
    engine = None

    def __init__(self, df: pd.DataFrame):
        self.df = df

    def write_snowflake(self, database: str, schema: str, table: str, sf_username: str = None, sf_password: str = None,
                        sf_account: str = None):
        """Uploads data from a pandas dataframe to an existing Snowflake table."""

        try:
            # initialize Snowflake connection and configure credentials
            self.initialize_snowflake(
                             database=database,
                             schema=schema,
                             sf_username=sf_username,
                             sf_password=sf_password,
                             sf_account=sf_account
                             )

            # create SQL Alchemy engine
            self.engine = create_engine(URL(
                user=self.sf_credentials['user'],
                password=self.sf_credentials['pass'],
                account=self.sf_credentials['acct'],
                database=database,
                schema=schema
            ))

            # calls method to write data in a pandas dataframe to an existing Snowflake table
            # will create a new snowflake table if the given table name does not exist
            self.df.to_sql(name=table, con=self.engine, index=False, if_exists='append', method=pd_writer)

            self.write_success = True

        # catch and log error
        except Exception as e:
            self.log_message = e
            self._logger.error(self.log_message)
            return False

        finally:
            # close connection
            if self.connection:
                self.connection.close()
            if self.engine:
                self.engine.dispose()

        if self.write_success:
            self.log_message = f"Successfully wrote to the {table} Snowflake table"
            self._logger.info(self.log_message)
            return True




import platform
import os
import yaml
import pandas as pd
from cloudy_warehouses.from_snowflake import SnowflakeReader
from cloudy_warehouses.to_snowflake import SnowflakeWriter
from cloudy_warehouses.copy_snowflake import SnowflakeCopier

if not os.getenv('CLOUDY_HOME'):
    if platform.system().lower() != 'windows':
        os.environ['CLOUDY_HOME'] = os.getenv('HOME')
    else:
        os.environ['CLOUDY_HOME'] = os.getenv('USERPROFILE')

# Create configuration
profiles_path = os.path.join(os.getenv("CLOUDY_HOME"), ".cloudy_warehouses/configuration_profiles.yml")
default_profiles_path: str = os.path.join(os.path.dirname(__file__), 'configurations/default_configuration_profiles.yml')

#  If the configuration path does not exist - then a default configuration will be created
if not os.path.exists(profiles_path):

    # Set the path for the default configuration if it does not exist
    cloudy_profiles = os.path.join(os.getenv("CLOUDY_HOME"), ".cloudy_warehouses")
    if not os.path.exists(cloudy_profiles):
        os.mkdir(cloudy_profiles)

    # Load the default configuration
    with open(default_profiles_path, 'r') as default_stream:
        profiles_configuration = yaml.safe_load(default_stream)

    # Write the default configuration
    with open(profiles_path, 'w') as stream:
        _ = yaml.dump(profiles_configuration, stream)


# Add ability to read from pandas
pd.read_snowflake = SnowflakeReader().read

# Add ability to list from pandas
pd.list_snowflake_tables = SnowflakeReader().list_tables

# Add ability to copy from pandas
pd.clone_snowflake = SnowflakeCopier().clone

# Add ability to copy empty from pandas
pd.clone_empty_snowflake = SnowflakeCopier().clone_empty



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="cloudy_warehouses",
    version="0.0.4.0",
    author="Hashmap, Inc",
    author_email="accelerators@hashmapinc.com",
    description="Cloudy Warehouses allows for stream-lined interaction between pandas and cloud data platform providers",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/hashmapinc/sales-and-marketing/pocs/sam-kohlleffel-poc/pandas-cloudy-extension",
    packages=setuptools.find_packages(),
    package_data={
        "cloudy_warehouses": ["configurations/default_configuration_profiles.yml"],
    },
    install_requires=[
        'pyarrow==0.17.1',
        'pandas==1.1.4',
        'pyyaml==5.3.1',
        'snowflake-connector-python==2.3.6',
        'snowflake-sqlalchemy==1.2.4',
        'SQLAlchemy==1.3.20'

    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Development Status :: 3 - Alpha"
    ],
    python_requires='>=3.7',
)
